package net.sf.jasperreports.engine.query;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URISyntaxException;
import java.util.Locale;
import java.util.Map;
import java.util.TimeZone;
import net.sf.jasperreports.engine.DefaultJasperReportsContext;
import net.sf.jasperreports.engine.JRDataSource;
import net.sf.jasperreports.engine.JRDataset;
import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JRValueParameter;
import net.sf.jasperreports.engine.JasperReportsContext;
import net.sf.jasperreports.engine.data.JsonDataSource;
import net.sf.jasperreports.engine.manager.RESTCallManager;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.http.client.utils.URIBuilder;

public class JsonQueryExecuter
  extends JRAbstractQueryExecuter
{
  private static final Log log = LogFactory.getLog(JsonQueryExecuter.class);
  protected static final String CANONICAL_LANGUAGE = "JSON";
  private JsonDataSource datasource;
  
  public JsonQueryExecuter(JasperReportsContext jasperReportsContext, JRDataset dataset, Map<String, ? extends JRValueParameter> parametersMap)
  {
    super(jasperReportsContext, dataset, parametersMap);
    parseQuery();
  }
  
  /**
   * @deprecated
   */
  public JsonQueryExecuter(JRDataset dataset, Map<String, ? extends JRValueParameter> parametersMap)
  {
    this(DefaultJasperReportsContext.getInstance(), dataset, parametersMap);
  }
  
  protected String getCanonicalQueryLanguage()
  {
    return "JSON";
  }
  
  protected String getParameterReplacement(String parameterName)
  {
    return String.valueOf(getParameterValue(parameterName));
  }
  
  public JRDataSource createDatasource()
    throws JRException
  {
    ByteArrayInputStream byteArrayInputStream = null;
    InputStream jsonInputStream = null;
    
    RESTCallManager restCallManager = new RESTCallManager();
    
    String urlString = getStringParameterOrProperty("REST_URL");
    String sourceFile = getStringParameterOrProperty("SOURCE_FILE");
    String reportType = getStringParameterOrProperty("reportType");
    String queryParams = getStringParameterOrProperty("queryParams");
    log.info("ReportType:" + reportType);
    
    URIBuilder uri = getURI(urlString, queryParams);
    try
    {
      if ((urlString != null) && (!"".equals(urlString)))
      {
        String jsonString = restCallManager.makeAsyncRESTcall(uri);
        if (StringUtils.isNotBlank(jsonString))
        {
          byte[] bytes = jsonString.getBytes();
          byteArrayInputStream = new ByteArrayInputStream(bytes);
          jsonInputStream = byteArrayInputStream;
          log.info("jsonInputStream " + jsonInputStream);
          if (jsonInputStream != null) {
            this.datasource = new JsonDataSource(jsonInputStream, getQueryString());
          }
        }
        else
        {
          log.info("No Data Found From REST Service ");
        }
      }
      else if ((sourceFile != null) && (!"".equals(sourceFile)))
      {
        log.info("sourceFile " + sourceFile);
        this.datasource = new JsonDataSource(sourceFile, getQueryString());
      }
      else
      {
        log.info("No Data Source Configured ");
      }
      log.info("datasource " + this.datasource);
      if (this.datasource != null)
      {
        String dateFormatPattern = getStringParameterOrProperty("net.sf.jasperreports.json.date.pattern");
        if (dateFormatPattern != null) {
          this.datasource.setDatePattern(dateFormatPattern);
        }
        String numberFormatPattern = getStringParameterOrProperty("net.sf.jasperreports.json.number.pattern");
        if (numberFormatPattern != null) {
          this.datasource.setNumberPattern(numberFormatPattern);
        }
        Locale jsonLocale = (Locale)getParameterValue("JSON_LOCALE", true);
        if (jsonLocale != null)
        {
          this.datasource.setLocale(jsonLocale);
        }
        else
        {
          String jsonLocaleCode = getStringParameterOrProperty("net.sf.jasperreports.json.locale.code");
          if (jsonLocaleCode != null) {
            this.datasource.setLocale(jsonLocaleCode);
          }
        }
        TimeZone jsonTimezone = (TimeZone)getParameterValue("JSON_TIME_ZONE", true);
        if (jsonTimezone != null)
        {
          this.datasource.setTimeZone(jsonTimezone);
        }
        else
        {
          String jsonTimezoneId = getStringParameterOrProperty("net.sf.jasperreports.json.timezone.id");
          if (jsonTimezoneId != null) {
            this.datasource.setTimeZone(jsonTimezoneId);
          }
        }
      }
      return this.datasource;
    }
    finally
    {
      try
      {
        if (jsonInputStream != null) {
          jsonInputStream.close();
        }
        if (byteArrayInputStream != null) {
          byteArrayInputStream.close();
        }
      }
      catch (IOException e)
      {
        e.printStackTrace();
      }
    }
  }
  
  private URIBuilder getURI(String urlString, String queryParams)
  {
    URIBuilder uri = null;
    try
    {
      uri = new URIBuilder(urlString);
    }
    catch (URISyntaxException e)
    {
      log.error("URI Builder Exception:" + e.getMessage());
      e.printStackTrace();
    }
    String[] params = queryParams.split(",");
    for (String key : params) {
      if (!"".equals(key))
      {
        String value = getStringParameterOrProperty(key);
        if (value != null) {
          uri.addParameter(key, value);
        }
      }
    }
    log.info("URI:" + uri);
    return uri;
  }
  
  public void close()
  {
    if (this.datasource != null) {
      this.datasource.close();
    }
  }
  
  public boolean cancelQuery()
    throws JRException
  {
    return false;
  }
}
